function [queries,boundaries,observations]=ray(x,epsilon,Nq,Nmaxblock,delta,filename);

% function [queries,boundaries,observations]=ray(x,epsilon,Nq,Nmaxblock,delta,filename);
%
% This function creates a sequence of queries and boundary estimates
% similar to the BZ algorithm, but now the samples are collected at random
% locations. It also creates a movie file according to the
% software specifications of the human active learning experiment.
%
% x         - True boundary location (-8:0.05:8)
% epsilon   - Noise level (epsilon<=0<0.05)
% Nq        - number of queries per block
% Nmaxblock - Number of blocks
% delta     - Boundary choice tolerance
% filename  - movie filename (e.g. 'movie.txt')
%
% queries      - list of query points
% boundaries   - list of boundary estimates
% observations - list of observated values (0=green)

fid=fopen(filename,'w');

% Write the heading of the movie file
fprintf(fid,'x %1.2f\r\n',x);
fprintf(fid,'epsilon %1.3f\r\n',epsilon);
fprintf(fid,'Nq %d\r\n',Nq);
fprintf(fid,'Nmaxblock %d\r\n',Nmaxblock);
fprintf(fid,'delta %1.3f\r\n',delta);

if nargin==6;
    alpha=epsilon;
end;

% l - List of possible shapes
% m - Number of possible shapes
l=-8:0.05:8;
m=length(l);

% Posterior vector
p=ones(1,m)/m;

% initialize the log variables
queries=zeros(1,Nq*Nmaxblock);
observations=zeros(1,Nq*Nmaxblock);
boundaries=zeros(1,Nq*Nmaxblock);

for j=1:Nmaxblock;    %run for Nmaxblocks (if we incorporate the tolerance about querying similar points this needs to be changed)
    for k=1:Nq;       %do Nq queries per block

        i=min(find(cumsum(p)>1/2));
        tau1=sum(p(i:m))-sum(p(1:i-1));
        tau2=sum(p(1:i))-sum(p(i+1:m));
        q_i=ceil(rand*length(l));   % index of the query we are going to make

        queries(k+Nq*(j-1))=l(q_i);

        y=xor(l(q_i)<x,rand<epsilon);     % noisy query
        observations(k+Nq*(j-1))=~y;      % observation - in this implementation I have a notation that is the reverse of yours

        p(1:q_i)=2*(1-alpha)*p(1:q_i)*(~y)+2*alpha*p(1:q_i)*y;        %Posterior update and normalization
        p(q_i+1:m)=2*alpha*p(q_i+1:m)*(~y)+2*(1-alpha)*p(q_i+1:m)*y;
        p=p/sum(p);
        
        boundaries(k+Nq*(j-1))=l(min(find(cumsum(p)>1/2)));
    end;
end;

% Create the movie in a human-like way
%parameters
min_pause_scrolling=18;
delta_pause_scrolling=8;
min_pause_querying=40;
delta_pause_querying=20;
min_pause_boundary=150;
delta_pause_boundary=100;
max_scroll_length=3;
min_number_scrolling_events=5;
delta_number_scrolling_events=5;


last_event=0;   %last query point (if not reset to zero)
time=0;         %time stamp
tolerance=inf*[1 1 1];   %Tolerance vector
for j=1:Nmaxblock;
    for k=1:Nq;
        
        time=time+min_pause_scrolling+rand*delta_pause_scrolling;    %pause in between scroll events

        while abs(queries(k+Nq*(j-1))-last_event)>max_scroll_length;   % maximum scrolling, about 3 "shape" units
            tmp3=last_event+3*sign(queries(k+Nq*(j-1))-last_event);
            tmp4=min_number_scrolling_events+delta_number_scrolling_events*rand;            %approximatly 5-10 mouse events per scrolling burst
            tmp=round((last_event+(tmp3-last_event)*(0:tmp4-1)/(tmp4-1))*20)/20;
            tmp2=round(time+(0:tmp4-1)/(tmp4-1));
            time=time+1;
            fprintf(fid,'scroll %1.2f %d\r\n',[tmp;tmp2]);
            last_event=tmp3;
            time=time+min_pause_scrolling+rand*delta_pause_scrolling;    %pause in between scroll events
        end;

        tmp4=min_number_scrolling_events+delta_number_scrolling_events*rand;            %approximatly 5-10 mouse events per scrolling burst
        tmp=round((last_event+(queries(k+Nq*(j-1))-last_event)*(0:tmp4-1)/(tmp4-1))*20)/20;
        tmp2=round(time+(0:tmp4-1)/(tmp4-1));
        time=time+1;
        fprintf(fid,'scroll %1.2f %d\r\n',[tmp;tmp2]);            
        last_event=queries(k+Nq*(j-1));

        time=time+min_pause_querying+rand*delta_pause_querying;       %pause in between scrolling and querying
        fprintf(fid,'query %1.2f %d %d\r\n',[queries(k+Nq*(j-1)) observations(k+Nq*(j-1)) round(time)]);
    end;
    time=time+min_pause_boundary+rand*delta_pause_boundary;   %time taken to choose a boundary
    fprintf(fid,'boundary %1.2f %d\r\n',[boundaries(k+Nq*(j-1)) round(time)]);
    last_event=0;
    tolerance=[tolerance(2:3) boundaries(k+Nq*(j-1))];
    if (max(tolerance)-min(tolerance))<=delta
        break;
    end;
end;

fclose(fid);
return;
